//  ------------------------------------------------------------------------
//
//  This file is part of the Intan Technologies RHD2000 Interface
//  Version 2.0
//  Copyright (C) 2013-2016 Intan Technologies
//
//  ------------------------------------------------------------------------
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <QtGui>
#include <iostream>

#include "qtincludes.h"

#include "cabledelaydialog.h"

using namespace std;

// Manual SPI cable delay configuration dialog.
// This dialog allows users to select fixed values that the FPGA uses to compensate
// for signal propagation delays in the SPI interface cables.

CableDelayDialog::CableDelayDialog(QVector<bool> &manualDelayEnabled, vector<int> &currentDelay, int numPorts, QWidget *parent) :
    QDialog(parent)
{
    QLabel *label1 = new QLabel(
                tr("The Intan Recording Controller can compensate for the nanosecond-scale time delays "
                   "resulting from finite signal velocities on the SPI interface cables.  "
                   "Each time the interface software is opened or the <b>Rescan Ports</b> button is "
                   "clicked, the software attempts to determine the optimum delay settings for each SPI "
                   "port.  Sometimes this delay-setting algorithm fails, particularly when using RHD2164 "
                   "chips which use a double-data-rate SPI protocol."));
    label1->setWordWrap(true);

    QLabel *label2 = new QLabel(
                tr("This dialog box allows users to override this algorithm and set delays manually.  "
                   "If a particular SPI port is returning noisy signals with large discontinuities, "
                   "try checking the manual delay box for that port and adjust the delay setting up or "
                   "down by one."));
    label2->setWordWrap(true);

    QLabel *label3 = new QLabel(
                tr("Note that the optimum delay setting for a particular SPI cable length will change if the "
                   "amplifier sampling rate is changed."));
    label3->setWordWrap(true);

    manualPortACheckBox = new QCheckBox("Set manual delay for Port A");
    manualPortBCheckBox = new QCheckBox("Set manual delay for Port B");
    manualPortCCheckBox = new QCheckBox("Set manual delay for Port C");
    manualPortDCheckBox = new QCheckBox("Set manual delay for Port D");
    manualPortECheckBox = new QCheckBox("Set manual delay for Port E");
    manualPortFCheckBox = new QCheckBox("Set manual delay for Port F");
    manualPortGCheckBox = new QCheckBox("Set manual delay for Port G");
    manualPortHCheckBox = new QCheckBox("Set manual delay for Port H");

    manualPortACheckBox->setChecked(manualDelayEnabled[0]);
    manualPortBCheckBox->setChecked(manualDelayEnabled[1]);
    manualPortCCheckBox->setChecked(manualDelayEnabled[2]);
    manualPortDCheckBox->setChecked(manualDelayEnabled[3]);
    manualPortECheckBox->setChecked(manualDelayEnabled[4]);
    manualPortFCheckBox->setChecked(manualDelayEnabled[5]);
    manualPortGCheckBox->setChecked(manualDelayEnabled[6]);
    manualPortHCheckBox->setChecked(manualDelayEnabled[7]);

    delayPortASpinBox = new QSpinBox();
    delayPortASpinBox->setRange(0, 15);
    delayPortASpinBox->setValue(currentDelay[0]);

    delayPortBSpinBox = new QSpinBox();
    delayPortBSpinBox->setRange(0, 15);
    delayPortBSpinBox->setValue(currentDelay[1]);

    delayPortCSpinBox = new QSpinBox();
    delayPortCSpinBox->setRange(0, 15);
    delayPortCSpinBox->setValue(currentDelay[2]);

    delayPortDSpinBox = new QSpinBox();
    delayPortDSpinBox->setRange(0, 15);
    delayPortDSpinBox->setValue(currentDelay[3]);

    delayPortESpinBox = new QSpinBox();
    delayPortESpinBox->setRange(0, 15);
    delayPortESpinBox->setValue(currentDelay[4]);

    delayPortFSpinBox = new QSpinBox();
    delayPortFSpinBox->setRange(0, 15);
    delayPortFSpinBox->setValue(currentDelay[5]);

    delayPortGSpinBox = new QSpinBox();
    delayPortGSpinBox->setRange(0, 15);
    delayPortGSpinBox->setValue(currentDelay[6]);

    delayPortHSpinBox = new QSpinBox();
    delayPortHSpinBox->setRange(0, 15);
    delayPortHSpinBox->setValue(currentDelay[7]);

    buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);

    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

    QHBoxLayout *portARow = new QHBoxLayout;
    portARow->addWidget(manualPortACheckBox);
    portARow->addStretch(1);
    portARow->addWidget(new QLabel(tr("Current delay:")));
    portARow->addWidget(delayPortASpinBox);

    QHBoxLayout *portBRow = new QHBoxLayout;
    portBRow->addWidget(manualPortBCheckBox);
    portBRow->addStretch(1);
    portBRow->addWidget(new QLabel(tr("Current delay:")));
    portBRow->addWidget(delayPortBSpinBox);

    QHBoxLayout *portCRow = new QHBoxLayout;
    portCRow->addWidget(manualPortCCheckBox);
    portCRow->addStretch(1);
    portCRow->addWidget(new QLabel(tr("Current delay:")));
    portCRow->addWidget(delayPortCSpinBox);

    QHBoxLayout *portDRow = new QHBoxLayout;
    portDRow->addWidget(manualPortDCheckBox);
    portDRow->addStretch(1);
    portDRow->addWidget(new QLabel(tr("Current delay:")));
    portDRow->addWidget(delayPortDSpinBox);

    QHBoxLayout *portERow = new QHBoxLayout;
    portERow->addWidget(manualPortECheckBox);
    portERow->addStretch(1);
    portERow->addWidget(new QLabel(tr("Current delay:")));
    portERow->addWidget(delayPortESpinBox);

    QHBoxLayout *portFRow = new QHBoxLayout;
    portFRow->addWidget(manualPortFCheckBox);
    portFRow->addStretch(1);
    portFRow->addWidget(new QLabel(tr("Current delay:")));
    portFRow->addWidget(delayPortFSpinBox);

    QHBoxLayout *portGRow = new QHBoxLayout;
    portGRow->addWidget(manualPortGCheckBox);
    portGRow->addStretch(1);
    portGRow->addWidget(new QLabel(tr("Current delay:")));
    portGRow->addWidget(delayPortGSpinBox);

    QHBoxLayout *portHRow = new QHBoxLayout;
    portHRow->addWidget(manualPortHCheckBox);
    portHRow->addStretch(1);
    portHRow->addWidget(new QLabel(tr("Current delay:")));
    portHRow->addWidget(delayPortHSpinBox);

    QVBoxLayout *controlLayout = new QVBoxLayout;
    controlLayout->addLayout(portARow);
    controlLayout->addLayout(portBRow);
    controlLayout->addLayout(portCRow);
    controlLayout->addLayout(portDRow);
    if (numPorts == 8) {
        controlLayout->addLayout(portERow);
        controlLayout->addLayout(portFRow);
        controlLayout->addLayout(portGRow);
        controlLayout->addLayout(portHRow);
    }

    QGroupBox *controlBox = new QGroupBox();
    controlBox->setLayout(controlLayout);

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(label1);
    mainLayout->addWidget(label2);
    mainLayout->addWidget(label3);
    mainLayout->addWidget(controlBox);
    mainLayout->addWidget(buttonBox);

    setLayout(mainLayout);

    setWindowTitle(tr("Manual SPI Cable Delay Configuration"));
}
